%% ========================================================================
%  MIGRATE MODEL: Version dec 2018
%--------------------------------------------------------------------------
% This modle has been developed thanks to the EC-founded Migrate Project WP3
% Contributions from
% Uros Markovic (ETH Zurich)
% Taouba Jouini (ETH Zurich)
% Dominic Gross (ETH Zurich)
% Taoufik Qoria (L2EP Lille)
% Guillaume Denis (RTE France)
%  ========================================================================



clear all

%% ========================================================================
%  Define simulation parameters
%  ========================================================================
%% Simulation setup (for the user)
%   number of converters
model.num = 3;

%   Select test case
model.event = '3-A-1';

%% Simulation Parameters (for the user - optional)
sim.T_start = 0;        % Simulation start time [s]
sim.T_stop = 2.5;         % Simulation end time [s]
sim.Sim_type = 'fixed'; % Simulation type: {fixed} or {variable} time step
sim.T_step = 1e-4;      % Simulation time step (if fixed) [s]
sim.Solver = 'Auto';    % Solver type: Auto, Discrete, Ode8, Ode5 - Ode1, Ode14x

controller=[1,2,5]; %1.2.5
for i=1:model.num
    switch controller(i)
        case 1 %grid forming droop
            %switches
            sync_mode(i) = 1;         %source of the frequency reference (1: nominal freq, 2: PLL, 3: VIM)
            sw_outer(i) = 1;          %type of outer control (1: droop, 2: dVOC, 3: matching)
            ctr_mode(i) = 1;       %1: use standard cascaded control structure, 2: direct control of modulation
            
            %enable the corresponding sybsystems
            droop_outer(i) = 1;
            dVOC_outer(i) = 0;
            match_outer(i) = 0;
            PLL_sync(i) = 0;
            VIM_sync(i) = 0;
            cascade_ctr(i) = 1;
            match_ctr(i) = 0;
            
            
        case 2 %grid forming dVOC
            sync_mode(i) = 1;         %source of the frequency reference (1: nominal freq, 2: PLL, 3: VIM)
            sw_outer(i) = 2;          %type of outer control (1: droop, 2: dVOC, 3: matching)
            ctr_mode(i) = 1;       %1: use standard cascaded control structure, 2: direct control of modulation
            
            %enable the corresponding sybsystems
            droop_outer(i) = 0;
            dVOC_outer(i) = 1;
            match_outer(i) = 0;
            PLL_sync(i) = 0;
            VIM_sync(i) = 0;
            cascade_ctr(i) = 1;
            match_ctr(i) = 0;
            
        case 3 %grid following droop with PLL
            %switches
            sync_mode(i) = 2;         %source of the frequency reference (1: nominal freq, 2: PLL, 3: VIM)
            sw_outer(i) = 1;          %type of outer control (1: droop, 2: dVOC, 3: matching)
            ctr_mode(i) = 1;       %1: use standard cascaded control structure, 2: direct control of modulation
            
            %enable the corresponding sybsystems
            droop_outer(i) = 1;
            dVOC_outer(i) = 0;
            match_outer(i) = 0;
            PLL_sync(i) = 1;
            VIM_sync(i) = 0;
            cascade_ctr(i) = 1;
            match_ctr(i) = 0;
            
            
        case 4 %grid following droop with VIM
            %switches
            sync_mode(i) = 3;         %source of the frequency reference (1: nominal freq, 2: PLL, 3: VIM)
            sw_outer(i) = 1;          %type of outer control (1: droop, 2: dVOC, 3: matching)
            ctr_mode(i) = 1;       %1: use standard cascaded control structure, -1: direct control of modulation
            
            %enable the corresponding subsystems
            droop_outer(i) = 1;
            dVOC_outer(i) = 0;
            match_outer(i) = 0;
            PLL_sync(i) = 0;
            VIM_sync(i) = 1;
            cascade_ctr(i) = 1;
            match_ctr(i) = 0;
            
            
        case 5 %matching control as reference
            %switches
            sync_mode(i) = 1;         %source of the frequency reference (1: nominal freq, 2: PLL, 3: VIM)
            sw_outer(i) = 3;          %type of outer control (1: droop, 2: dVOC, 3: matching)
            ctr_mode(i) = 1;       %1: use standard cascaded control structure, 2: direct control of modulation
            
            %enable the corresponding subsystems
            droop_outer(i) = 0;
            dVOC_outer(i) = 0;
            match_outer(i) = 1;
            PLL_sync(i) = 0;
            VIM_sync(i) = 0;
            cascade_ctr(i) = 1;
            match_ctr(i) = 0;
            
        case 6 %direct matching control of the modulation
            %switches
            sync_mode(i) = 1;         %source of the frequency reference (1: nominal freq, 2: PLL, 3: VIM)
            sw_outer(i) = 1;          %type of outer control (1: droop, 2: dVOC, 3: matching)
            ctr_mode(i) = 2;       %1: use standard cascaded control structure, 2: direct control of modulation
            
            %enable the corresponding subsystems
            droop_outer(i) = 0;
            dVOC_outer(i) = 0;
            match_outer(i) = 0;
            PLL_sync(i) = 0;
            VIM_sync(i) = 0;
            cascade_ctr(i) = 0;
            match_ctr(i) = 1;
    end
end


%% Processing
%   Eliminate unnecessary Simulink warnings
warning('off','Simulink:Masking:Image_IgnoreLocationInAutoscale');

%   Add paths
addpath(genpath([pwd,'/Images']));
addpath(genpath([pwd,'/Include']));

model.name = ['test_case_', num2str(model.num), '_VSC_R2017a'];
open_system(model.name);

% General convention: Separate lines for separate VSCs, separate column for
% separate phase



%% Nominal Parameters
% VSC
switch model.num
    case 1
        Pn = 1000 * 1e6;            % Nom. VSC power [W]
        Pb = 1000  * 1e6;
    case 2
        Pn = [125;1000] * 1e6;      % Nom. VSC power [W]
        Pb = [125;1000] * 1e6;
    case 3
        Pn = [250;500;1000] * 1e6;      % Nom. VSC power [W]
        Pb = [250;500;1000] * 1e6;
end

%% Base Values

Ub = 320 * 1e3;
fb = 50;
wb = 2*pi*fb ;
wb_pu = 1;
cosPhi_b = cos(atan2(0.3,1));
Sb = (Pb ./ cosPhi_b);
Tb = (Sb ./ wb);
Vb = Ub;
Ib = (Sb ./ Ub);
Zb = (Vb ./ Ib);
Lb = (Zb ./ wb);
Cb = 1 ./ (Zb .* wb);
fg = fb; % Grid frequency [Hz]







cosPhi_n = cos(atan2(0.3, 1));  % Nom. cos(phi)
Un = 320 * 1e3;                 % Nom. phase-to-phase VSC voltage [V]
fn = 50;                        % Nom. frequency [Hz]
wn = 2*pi*fn;                   % Nom. angular velocity [rad/s]
Sn = Pn / cosPhi_n;             % Nom. apparent power [VA]
Un_dc = 640*1e3;                % Nom. DC voltage [V]

% Low-pass filter
Rf_pu = 0.005 * ones(model.num,1);             % Low-pass filter resistance Rf [p.u.]
Lf_pu = 0.15 * ones(model.num,1);             % Low-pass filter inductance Lf [p.u.]
Cf_pu = 0.066* ones(model.num,1);             % Low-pass filter capacitance Cf [p.u.]


% Loads

switch model.event
    case '2-A-1'
        P_load = 1200 * 1e6;  % Load active power P_load [W]
        Q_load = 0;          % Load reactive power Q_load [VAr]
    case '1-A-1'
        P_load = 500 * 1e6;  % Load active power P_load [W]
        Q_load = 0;          % Load reactive power Q_load [VAr]
    otherwise %In these cases the load is disconnected by a breaker.
        P_load = 500 * 1e6;        % Load active power P_load [W]
        Q_load = 0;          % Load reactive power Q_load [VAr]
end
% ---------------------- %
DeltaP_load = 100 * 1e6; % Delta load active power [W]
DeltaQ_load = 100 * 1e6;            % Delta load reactive power [VAr]

% Transformers
R_transformer_pu = 0.005 * Sb ./ Sn;    % Transformer resistances [p.u]
L_transformer_pu = 0.15 * Sb ./ Sn;     % Transformer inductances [p.u]

R_transformer = R_transformer_pu .* Zb;
L_transformer = L_transformer_pu .* Lb;

% Lines
r_line = 0.03;                      % Line resistance [Ohm/km]
x_line = 0.3;                       % Line inductance [Ohm/km]
l_line = x_line / wn;
c_line = 10e-9;                    % Line shunt capacitance [F/km]
line_len = [50 50 25 100 125];      % Line lengths [km]





%% Matrix Transformations
M_clarke = sqrt(2/3) * [   1     ,    -1/2     ,    -1/2     ;...
    0     ,  sqrt(3)/2  , -sqrt(3)/2  ;...
    1/sqrt(2)  ,  1/sqrt(2)  ,  1/sqrt(2) ];

M_clarke_inv = sqrt(2/3) * [   1   ,    0        ,  sqrt(2)/2  ;...
    -1/2  ,  sqrt(3)/2  ,  sqrt(2)/2  ;...
    -1/2  ,  -sqrt(3)/2 ,  sqrt(2)/2 ];



%% Upper Control Parameters
inner_loop = 'Taoufik';
if strcmp(model.event,'1-B-1-following')
    inner_loop = 'Uros';
elseif size(controller,2)==1 && controller(1,1)==4
    inner_loop = 'Uros';
end
switch inner_loop
    case 'Taoufik'
        % Voltage PI controller
        Tr_v = 50 * 1e-3;                  % Time response [s]
        D_v = 0.707;                       % Damping factor
        Kp_v = 0.52;
        Ki_v = 1.161022;
        Kffv = 1;                        % Voltage feed forward (enabled)
        Ti_v = Kp_v / Ki_v;
        
        % Current PI controller
        Tr_i = 5 * 1e-3;                   % Time response [s]
        D_i = 0.707;                       % Damping factor
        Kp_i = 0.738891;
        Ki_i = 1.19;
        Kffi = 1;                          % Current feed forward (enabled)
        Ti_i = Kp_i / Ki_i;
    case 'Uros'
        % Voltage PI controller
        Tr_v = 50 * 1e-3;                  % Time response [s]
        D_v = 0.707;                       % Damping factor
        Kp_v = 0.59;
        Ki_v = 736;
        Ti_v = Kp_v / Ki_v;
        
        % Current PI controller
        Tr_i = 5 * 1e-3;                   % Time response [s]
        D_i = 0.707;                       % Damping factor
        Kp_i = 1.27;
        Ki_i = 14.3;
        Kffi = 0;                         % Current feed forward (disabled)
        Kffv = 1;                         % Voltage feed forward (enabled)
        Ti_i = Kp_i / Ki_i;
end

% Droop controller
mp = 0.02 * ones(1,model.num);
nq = 0.000 * ones(1,model.num);

Tr_PLL=5e-3;
wn=5/Tr_PLL;
zeta=1;
Ti_PLL = 2*zeta/wn;
kp_PLL =-Ti_PLL*wn^2./Ub;

% DC BUS control
Cdc = 2e-3*Sb/(1000 * 1e6/cos(atan2(0.3,1)));
Tr_dc=100e-3;

% Virtual resistor
K_RV = 0.09;
W_RV = 16.66;

% Lead compensator
T2 = 0.0111;
T1 = 0.0333;


for i=1:model.num
    switch controller(i)
        case 5
            Ki_dc(i) = 0;    % deactivate the Integral controller of the DC side
            kp_dc(i) = 2;
            idcstar_enable(i)=1; %enable idcstar computation from P_set
            K_theta(i)=0.008;
            Kavr(i)=0; %disabled the matched AVR, the underlying voltage controller is used
            theta_0 = 0; % Replace by ICI_Theta in model
        case 6
            Ki_dc(i) = 0;
            kp_dc(i) = 0.1;
            idcstar_enable(i)=1; %enable idcstar computation from P_set
            % AVR Gain for amplitude control
            Kavr(i) = 0.01;
            
            % Initial Angle Theta_0
            theta_0 = 0; % Replace by ICI_Theta in model
            % Angle-P-gain
            K_theta = 200/Un_dc;
        otherwise
            wndc=5/Tr_dc;
            zeta=1;
            Ki_dc(i) = wndc*wndc*Cdc(i);
            kp_dc(i) =2*zeta*wndc*Cdc(i);
            idcstar_enable(i)=0; %do not use a feedforward term for idc
            K_theta(i)=0;
            Kavr(i)=0;
            theta_0=0;
    end
end

%dVOC gain
eta=400*10/6;
alpha=12*6/10; %alpha=12 de base

% Virtual impedance
Rv_pu = 0;
Lv_pu = 0.2;

% PLL PI parameters (for grid-following VSCs and droop+inertia)
% PLL_Ki = 0.96;
% PLL_Kp = 0.96/90e-3;
PLL_Ki = 4.69;
PLL_Kp = 0.4; %0.04

% Fast PLL PI parameters (only for measurement!)
FastPLL_Ki = 6634.6;
FastPLL_Kp = 114;

% Virtual induction machine emulator
f0_vim = 50;              % Initial rotor frequency [Hz]
w0_vim = 2*pi*f0_vim / wb;  % Initial angular rotor speed [p.u]
H_vim = 5.04;        % Inertia constant [s]
J_vim = 2 * H_vim * Sn * 1e-6 / (wb^2);  % Momentum of inertia
switch f0_vim
    case 50
        D_vim = 10 * ones(model.num,1);         % Damping constant [Nm/s]
    case 49.9
        D_vim = 2.7 * ones(model.num,1);         % Damping constant [Nm/s]
end

Rr_vim_pu = 0.0005;
Lr_vim_pu = 0.05;
Lm_vim_pu = 0.6;

Rr_vim = Rr_vim_pu * Zb;
Lr_vim = Lr_vim_pu * Lb;
Lm_vim = Lm_vim_pu * Lb;


% Control parameters
%   PD controller (w_slip)
Kp_vim = Rr_vim ./ Lr_vim;
Kd_vim = 0.001;
Nd_vim = 100;
%   Transfer function (Te)
Ke_vim_num = 3/2.*Rr_vim.*Lm_vim.^2;
Ke_vim_den = [Lr_vim.^2 , Rr_vim.*Lr_vim];
%   Inertia emulation (Delta_wr)
Inertia_vim_num = 1;
Inertia_vim_den = [J_vim , D_vim];

% VIM Iq/Id saturation limits
Iqd_sat = 0.25;    % Saturation current is 100% of VSC nominal current

% Switching time of a VIM
T_sync_switch = 0.01;

switch model.num
    case 1
        %% Simulation Events
        Tbr_grid = sim.T_stop + 1;
        Tbr_load = sim.T_stop + 1;
        Tsw_load = [sim.T_stop + 1, sim.T_stop + 2];
        Tsw_Vref = sim.T_stop + 1;
        Tsw_Pref = [sim.T_stop + 1, sim.T_stop + 2];
        
        Delta_Pload = [0, 0];
        Delta_Qload = [0, 0];
        Delta_Pref = [0, 0];
        Delta_Vref = 0;      % Reference change (dip and recovery in [p.u.])
        
        
        switch model.event
            case '1-A-1'
                % Event 1-A-1: Grid disconnection at t=0.5 s,
                %              load variation of +100MW at t=1.5s,
                %              load variation of -100MVA at t=3s
                SCR = 20;
                Tbr_grid = 0.5;
                Tsw_load = [1.5, 3];
                Delta_Pload = [100, 0]*1e6;
                Delta_Qload = [0, -100]*1e6;
                Tshrt = sim.T_stop+1;
                Tbr_dbl_1 = sim.T_stop+1;
                Tbr_dbl_2 = sim.T_stop+1;
                
            case '1-B-1'
                % Event 1-B-1: Strong grid; no load,
                %              power reference change +1000MW at t=1.5s,
                %              power reference change -1000MW at t=3s
                SCR = 20;
                Tbr_load = 0;
                Tsw_Pref = [2, 3,4];
                Delta_Pref = [800, -800, 0].*1e6./Sb;
                Tsw_Vref = [0.5, 1,1.5];
                Delta_Vref = [-16, 16, 0].*1e3./Vb;
                Tshrt = sim.T_stop+1;
                Tbr_dbl_1 = sim.T_stop+1;
                Tbr_dbl_2 = sim.T_stop+1;

            case '1-B-1-following'
                % Event 1-B-1: Strong grid; no load,
                %              power reference change +200MW at t=1.5s,
                %              power reference change -200MW at t=3s
                SCR = 20;
                Tbr_load = 0;
                Tsw_Pref = [1.5, 3];
                Delta_Pref = [200, -200].*1e6./Sb;
                Tshrt = sim.T_stop+1;
                Tbr_dbl_1 = sim.T_stop+1;
                Tbr_dbl_2 = sim.T_stop+1;                
                
            case '1-B-2'
                % Event 1````````````````````````````````````````````````````````````-B-2: Strong grid; no load,
                %              voltage reference change -16kV at t=1.5s
                SCR = 20;
                Tbr_load = 0;
                Tsw_Vref = 1.5;
                Delta_Vref = -16*1e3/Vb;
                Tshrt = sim.T_stop+1;
                Tbr_dbl_1 = sim.T_stop+1;
                Tbr_dbl_2 = sim.T_stop+1;
                
            case '1-B-3'
                % Event 1-B-3: Weak grid; no load,
                %              power reference change +P_reachable at t=1.5s,
                %              power reference change -P_reachable at t=3s
                SCR = 1;
                Tbr_load = 0;
                Tsw_Pref = [1.5, 3];
                switch controller
                    case 1
                        Delta_Pref = [800, -800]*1e6/Sb;
                    case 2
                        Delta_Pref = [800, -800]*1e6/Sb;
                    case 5
                        Delta_Pref = [900, -900]*1e6/Sb;
                    otherwise
                        Delta_Pref = [1000, -1000]*1e6/Sb;  % should be P_reachable
                end
                Tshrt = sim.T_stop+1;
                Tbr_dbl_1 = sim.T_stop+1;
                Tbr_dbl_2 = sim.T_stop+1;
                
            case '1-B-4'
                % Event 1-B-4: Weak grid; no load,
                %              voltage reference change -16kV at t=1.5s
                SCR = 1;
                Tbr_load = 0;
                Tsw_Vref = 1.5;
                Delta_Vref = -16*1e3/Vb;
                Tshrt = sim.T_stop+1;
                Tbr_dbl_1 = sim.T_stop+1;
                Tbr_dbl_2 = sim.T_stop+1;
                
            case '2-A-1'
                % Event 1-A-1: Short circuit in double line at t=1.5s,
                %              line opens at 1.65s
                SCR = 20;
                Tbr_grid = 0; %disable the direct connection to the grid
                Tbr_dbl_1 = [0 sim.T_stop+1]; %connect to grid via line 1
                Tbr_dbl_2 = [0 1.65];   %connect to grid via line 1
                Tshrt = 1.5; %short circuit at 1.5s
                
        end
        
% "Dispatcher" Set Points
switch model.event
    case '1-A-1'
        P_set = [0.45;0;0];                 % Active power set point for VSC [p.u]
        Q_set = [56e6/Sb(1);0;0];                   % Reactive power set point for VSC [p.u]
        Eg_set = [1.0092;1;1];                  % RMS voltage set point for VSC [p.u]
        Theta_E0 = [3 ;0;0]*pi/180; 
    case '1-B-1'
        P_set = [0;0;0];                 % Active power set point for VSC [p.u]
        Q_set = [0;0;0];                    % Reactive power set point for VSC [p.u]
        Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u]
        Theta_E0 = [0;0;0];
    case '1-B-1-following'
        P_set = [0.5;0;0];                 % Active power set point for VSC [p.u]
        Q_set = [0;0;0];                    % Reactive power set point for VSC [p.u]
        Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u]   
        Theta_E0 = [0;0;0];
    case '1-B-2'
        P_set = [0;0;0];                  % Active power set point for VSC [p.u]
        Q_set = [0;0;0];                    % Reactive power set point for VSC [p.u]
        Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u]
         Theta_E0 = [0;0;0];
    case '1-B-3'
        P_set = [0;0;0];                 % Active power set point for VSC [p.u]
        Q_set = [0;0;0];                    % Reactive power set point for VSC [p.u]
        Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u]
         Theta_E0 = [0;0;0];
    case '1-B-4'
        P_set = [0;0;0];                  % Active power set point for VSC [p.u]
        Q_set = [0;0;0];                    % Reactive power set point for VSC [p.u]
        Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u]
         Theta_E0 = [0;0;0];
    case '2-A-1'
        P_set = [0.55;0;0];                 % Active power set point for VSC [p.u]
        Q_set = [0;0;0];                   % Reactive power set point for VSC [p.u]
        Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u
         Theta_E0 = [0;0;0];
end

    case 3
        for i=1:model.num
            Tsw_Vref(i,:) = sim.T_stop + 1;
            Tsw_Pref(i,:) = [sim.T_stop + 1, sim.T_stop + 2];
            Delta_Pref(i,:) = [0, 0];
            Delta_Vref(i,:) = 0;      % Reference change (dip and recovery in [p.u.])
        end
        
        switch model.event




            case '3-A-1'
                % Event 3-A-1: Loss of the 500MW Inverter
                Tbr_500MW = 1; %Loss of the 500MW Inverter
                Tbr_line25 = sim.T_stop + 1; % No line opening
                Tbr_dbl100 = sim.T_stop + 1; % No line opening
                Tshrt = sim.T_stop + 1; %No short circuit 
                
%                 Tsw_Pref(1,:) = [1.5, 3];
%                 Delta_Pref(1,:)= [0.2, -0.2];
             
                % Result from LF file (Exemple)
%                 P_set = [0.6226;0.6226;0.6226];                 % Active power set point for VSC [p.u]
%                 Q_set = [0.0084;0.0084;0.0084];                   % Reactive power set point for VSC [p.u]
%                 Eg_set = [1;1;1];                  % RMS voltage set point for VSC [p.u
%                 Theta_E0 = [5.36 ;5.36;5.36]*pi/180;         %rad
%                 SCR = 20;                         
                
                % Result from LF file (3-node)
                P_set = [162.5e6/Sb(1);320.54e6/Sb(2);650.01e6/Sb(3)];                 % Active power set point for VSC [p.u]
                Q_set = [50e6/Sb(1);-10.58e6/Sb(2);-0.92e6/Sb(3)];                   % Reactive power set point for VSC [p.u]
                Eg_set = [1.0269;1;1];                  % RMS voltage set point for VSC [p.u
                Theta_E0 = [-2.78 ;0;0.13]*pi/180;         %rad
                SCR = 20;       
                
                
                
                

            case '3-A-2'
                % Event 3-A-1: Loss of the 25 km line, short circuit on dbl
                % line and opening of one defaulted line
                Tbr_500MW = sim.T_stop + 1; %No inverter loss
                Tbr_line25 = 0.5; %Open the 25km-line between 250MW and 1GW inverters
                Tbr_dbl100 = 1.650; % Open one of the dbl 100km-line between 250MW and 500 MW inverters 
                Tshrt = 1.5; %short circuit at 1.5s
             
 

                P_set = [162.5e6/Sb(1);320.54e6/Sb(2);650.01e6/Sb(3)];                 % Active power set point for VSC [p.u]
                Q_set = [50e6/Sb(1);-10.58e6/Sb(2);-0.92e6/Sb(3)];                   % Reactive power set point for VSC [p.u]
                Eg_set = [1.0269;1;1];                  % RMS voltage set point for VSC [p.u
                Theta_E0 = [-2.78 ;0;0.13]*pi/180;         %rad
                SCR = 20;       

            case '3-B-1'
            case '3-B-2'  
                
            otherwise 
                SCR = 20;
        end
end

% Grid parameters
Usrc0_pu = 1;
X_grid_pu = Usrc0_pu/SCR;    % Grid inductance [p.u]
R_grid_pu = X_grid_pu/10;         % Grid resistance [p.u]
L_grid_pu = X_grid_pu;

R_grid = R_grid_pu * Zb;
L_grid = L_grid_pu * Lb;
X_grid = L_grid * wb;
Eg_set_dq0 = kron([1,0,0],Eg_set);  

% Breaker parameters
R_breaker = 0.001;      % Breaker resistance [Ohm]
R_breaker_snb = 1e6;    % Snubber resistance [Ohm]
C_breaker_snb = inf;    % Snubber capacitance [F]

%% Initialization
% Grid voltage
Eg0_d =Eg_set; %1*ones(1,model.num)';
Eg0_q = 0*ones(1,3)';
Eg0_dq = [Eg0_d'; Eg0_q'; zeros(1,3)]';


% Load eqivalent
% R_load = Ub.^2 / P_load;
% L_load = Ub.^2 / (wb*Q_load);
% Z_load = sqrt(R_load^2 + (wb*L_load)^2);
% R_load_pu = R_load ./ Zb;
% L_load_pu = L_load ./ Lb;

%P0 = (Eg0_d.^2+Eg0_q.^2)./((R_load_pu + R_transformer_pu).^2 + (L_transformer_pu*wb_pu).^2).*(R_load_pu + R_transformer_pu);
%Q0 = (Eg0_d.^2+Eg0_q.^2)./((R_load_pu + R_transformer_pu).^2 + (L_transformer_pu*wb_pu).^2).*(L_transformer_pu*wb_pu);
P0=P_set;
Q0=Q_set;



% Ig
Ig0_d = (P0 .* Eg0_d + Q0 .* Eg0_q) ./ (Eg0_d.^2+Eg0_q.^2); %
Ig0_q = (P0 .* Eg0_q - Q0 .* Eg0_d) ./ (Eg0_d.^2+Eg0_q.^2); %Initial current in the connexion inductance
Ig0_dq = [Ig0_d'; Ig0_q' ; zeros(1,3)]';

Is0_d = Ig0_d - Eg0_q .* (Cf_pu * wb_pu); %% Irms
Is0_q = Ig0_q + Eg0_d .* (Cf_pu * wb_pu);
Is0_dq = [Is0_d'; Is0_q'; zeros(1,3)]';

% Vconv
Vm0_d = (Rf_pu .* Is0_d - Lf_pu * wb_pu .* Is0_q) + Eg0_d;
Vm0_q = (Lf_pu * wb_pu .* Is0_d + Rf_pu .* Is0_q) + Eg0_q;
Vm0_dq = [Vm0_d'; Vm0_q'; zeros(1,3)]';

% Convert to abc values

for i = (1:model.num)
theta0 = Theta_E0(i);
M_transform_inv = sqrt(2/3) * [   sin(theta0)     ,    cos(theta0)     ,    1/sqrt(2)     ;...
    sin(theta0-2*pi/3)     ,  cos(theta0-2*pi/3)  , 1/sqrt(2)  ;...
    sin(theta0+2*pi/3)     ,  cos(theta0+2*pi/3)  ,  1/sqrt(2) ];
%
Eg0_abc(i,:) = (M_transform_inv * Eg0_dq(i,:)')';
Vm0_abc(i,:) = (M_transform_inv * Vm0_dq(i,:)')';
Ig0_abc(i,:) = (M_transform_inv * Ig0_dq(i,:)')';
Is0_abc(i,:)= (M_transform_inv * Is0_dq(i,:)')';
end
Ws0 = wb;

%% Initial Conditions of Integrators (ICI)
% Current through filter inductance
ICI.Lf = Is0_abc / 1000;  %[kA]

% Voltage across filter capacitance
ICI.Cf = Eg0_abc / 1000;  %[kV]

% Phase of the voltage E0
ICI.Theta = Theta_E0;

% Current through the line
ICI.Line = Ig0_abc / 1000;  %[kA]

% d-axis current of the voltage-loop PI control
ICI.VoltagePI_d = ( Is0_d - Ig0_d .* Kffi + Eg0_q .* Cf_pu' * Ws0 / wb ) ./ Kp_v;

% q-axis current of the voltage-loop PI control
ICI.VoltagePI_q = (Is0_q - Ig0_q .* Kffi - Eg0_d .* Cf_pu' * Ws0 / wb ) ./ Kp_v;

% d-axis voltage of the current-loop PI control
ICI.CurrentPI_d = ( Vm0_d - Eg0_d .* Kffv + Is0_q .* Lf_pu' * Ws0 / wb ) ./ Kp_i;

% q-axis voltage of the current-loop PI control
ICI.CurrentPI_q = (Vm0_q - Eg0_q .* Kffv - Is0_d .* Lf_pu' * Ws0 / wb ) ./ Kp_i;

% Igdq0 for virtual damper
ICI.Damping_Igdq0=Ig0_dq;


%% Initial Output of Time Delays (IOTD)

% Initial output of the converter's current feedback time-delay
IOTD.Ic = Is0_abc / 1000;  %[kA]

% Initial output of the upper controller's current feedback time-delay
IOTD.Is = Ig0_abc / 1000;  %[kA]

% Initial output of the converter's voltage time-delay
IOTD.Vs = Eg0_abc / 1000;  %[kV]

%% Time Delays
Tdelay_Is = 100e-6;
Tdelay_Ig = 100e-6;
Tdelay_Eg = 100e-6;
Tdelay_vdc = 100e-6;
Isat_max = 1;
Isat_min = -1;
wc = 0.1*wb;

%% TVI
I_max_pu=1.2;
I_thresh_pu=1 ;  % Output current limiting threshold to activate DZ0_vi (pu)
DX_DR=5; %DX_vi/DR_vi
Rc_pu = 0.005;
Lc_pu = 0.15;
Kp_Rvi_pu=((-2*(I_max_pu-I_thresh_pu)*(Rc_pu+Lc_pu*DX_DR))+sqrt((2*(I_max_pu-I_thresh_pu)*(Rc_pu+Lc_pu*DX_DR))^2-4*(I_max_pu-I_thresh_pu)^2*(1+DX_DR^2)*(Rc_pu^2+Lc_pu^2-(1/I_max_pu)^2)))/(2*(I_max_pu-I_thresh_pu)^2*(1+DX_DR^2));
kFF_TVI = 0;
%% dVOC parameters and rotation matrices

%initial dVOC angle
theta0 = 0*4.774/180*pi;


%Rotation matrix R_0 for the dVOC reference voltage
R_0 = [cos(theta0), -sin(theta0);
    sin(theta0),  cos(theta0)];

%90-degree rotation matrix J
J = [0 -1;1 0];

%Rotation matrix R(kappa) for mixed droop depending on the L/R ratio of the grid
rho = (L_transformer(1))/(R_transformer(1));
kappa = atan(rho*2*pi*50);


R_kappa = [cos(kappa) -sin(kappa);
    sin(kappa)  cos(kappa)];




%% Set Solver Parameters
hCs = getActiveConfigSet(model.name);
hS = hCs.getComponent('Solver');
hS.StartTime = num2str(sim.T_start);
hS.StopTime = num2str(sim.T_stop);
switch sim.Sim_type
    case 'fixed'
        hS.SolverType = 'Fixed-step';
        solverPrefix = 'FixedStep';
        hS.FixedStep = num2str(sim.T_step);
    case 'variable'
        hS.SolverType = 'Variable-step';
        solverPrefix = 'VariableStep';
end
hS.Solver = [solverPrefix, sim.Solver];

%% Set VSC Mask Parameters

for i = (1:model.num)
    set_param([model.name,'/VSC Subsystem ',num2str(i)],...
        'ind', num2str(i),...
        'Pn', num2str(Pn(i) / 1e6),...
        'cosPhi_n', num2str(cosPhi_n),...
        'Un', num2str(Un / 1e3),...
        'Un_dc', num2str(Un_dc / 1e3),...
        'fn', num2str(fn),...
        'Rf_pu', num2str(Rf_pu(i)),...
        'Lf_pu', num2str(Lf_pu(i)),...
        'Cf_pu', num2str(Cf_pu(i)),...
        'Kp_v', num2str(Kp_v),...
        'Ki_v', num2str(Ki_v),...
        'D_v', num2str(D_v),...
        'Tr_v', num2str(Tr_v),...
        'Kff_v', num2str(Kffv),...
        'Kp_i', num2str(Kp_i),...
        'Ki_i', num2str(Ki_i),...
        'D_i', num2str(D_i),...
        'Tr_i', num2str(Tr_i),...
        'Kff_i', num2str(Kffi),...
        'ICI_Lf', ['[ ',num2str(ICI.Lf(i,:)),' ]'],...
        'ICI_Cf', ['[ ',num2str(ICI.Cf(i,:)),' ]'],...
        'ICI_Theta', num2str(ICI.Theta(i)),...
        'ICI_Line', ['[ ',num2str(ICI.Line(i,:)),' ]'],...
        'ICI_VoltagePI_d', ['[ ',num2str(ICI.VoltagePI_d(i)),' ]'],...
        'ICI_VoltagePI_q', ['[ ',num2str(ICI.VoltagePI_q(i)),' ]'],...
        'ICI_CurrentPI_d', ['[ ',num2str(ICI.CurrentPI_d(i)),' ]'],...
        'ICI_CurrentPI_q', ['[ ',num2str(ICI.CurrentPI_q(i)),' ]'],... 
        'ICI_Damping_Igdq0', ['[ ',num2str(ICI.Damping_Igdq0(i,:)),' ]'],...
        'Eg_set_dq0', ['[ ',num2str(Eg_set_dq0(i,:)),' ]'],...
        'Delta_Theta', num2str(0),...
        'Delta_Vref', ['[ ',num2str(Delta_Vref(i,:)),' ]'],...
        'Tsw_Vref', ['[ ',num2str(Tsw_Vref(i,:)),' ]'],...
        'Tdelay_Is', num2str(Tdelay_Is),...
        'Tdelay_Ig', num2str(Tdelay_Ig),...
        'Tdelay_Eg', num2str(Tdelay_Eg),...
        'P0', num2str(P0(i)),...
        'Q0', num2str(Q0(i)),...
        'P_set', num2str(P_set(i)),...
        'Q_set', num2str(Q_set(i)),...
        'mp', num2str(mp(i)),...
        'nq', num2str(nq(i)),...
        'Isat_max', num2str(Isat_max),...
        'Isat_min', num2str(Isat_min),...
        'wc', num2str(wc),...
        'Delta_Pref', ['[ ',num2str(Delta_Pref(i,:)),' ]'],...
        'Tsw_Pref', ['[ ',num2str(Tsw_Pref(i,:)),' ]'],...
        'Vm0_d', num2str(Vm0_d(i)),...
        'Vm0_q', num2str(Vm0_q(i)),...
        'Zb' , num2str(Zb(i)),...
        'Lb' , num2str(Lb(i)),...
        'Cb' , num2str(Cb(i)),...
        'Cdc', num2str(Cdc(i)),...
        'Ki_dc', num2str(Ki_dc(i)),...
        'kp_dc', num2str(kp_dc(i)),...
        'idcstar_enable', num2str(idcstar_enable(i)),...
        'K_theta', num2str(K_theta(i)),...
        'Kavr', num2str(Kavr(i)),...
        'eta', num2str(eta),...
        'alpha',num2str(alpha),....
        'sync_mode',num2str(sync_mode(i)),...
        'sw_outer',num2str(sw_outer(i)),...
        'ctr_mode',num2str(ctr_mode(i)),...
        'droop_outer',num2str(droop_outer(i)),...
        'dVOC_outer',num2str(dVOC_outer(i)),...
        'match_outer',num2str(match_outer(i)),...
        'PLL_sync',num2str(PLL_sync(i)),...
        'VIM_sync',num2str(VIM_sync(i)),...
        'cascade_ctr',num2str(cascade_ctr(i)),...
        'match_ctr',num2str(match_ctr(i)),...
        'Inertia_vim_den',['[ ',num2str(Inertia_vim_den(i,:)),' ]'],...
        'Ke_vim_num',['[ ',num2str(Ke_vim_num(i,:)),' ]'],...        
        'Ke_vim_den',['[ ',num2str(Ke_vim_den(i,:)),' ]'],...
        'Sb',num2str(Sb(i)),...
        'Pb',num2str(Pb(i)),...
        'Ib',num2str(Ib(i)),...
                'Kp_vim',num2str(Kp_vim(i)));
end
%% ========================================================================